// 
//  PIC時計 （NTP自動時刻修正機能）
//
//  WROOM-02のWIFIでNTPサーバに接続し自動的に時刻を修正するPICプログラム
//  詳しくは下記WEBページを参照してください。
//  http://bmicom.dmsk.com/wroom-02/pic-clock
//
//  Created on 2016/06   (c)SpaGetty
// 
//  File:   main.c   プログラム本体
// 
// 使用PIC PIC18F25K22
//                          ┏━━∪━━┓
//              MCLR    ->  ┫1       28┣  <->  RB7 7セグdp
// 7セグ1桁目   RA0    <->  ┫2       27┣  <->  RB6 7セグF
// 7セグ2桁目   RA1    <->  ┫3       26┣  <->  RB5 7セグE
// 7セグ3桁目   RA2    <->  ┫4       25┣  <->  RB4 7セグC
// 7セグ4桁目   RA3    <->  ┫5       24┣  <->  RB3 7セグA
// 7セグコロン  RA4    <->  ┫6       23┣  <->  RB2 7セグG
// おしらせLED  RA5    <->  ┫7       22┣  <->  RB1 7セグB
//              Vss     ->  ┫8       21┣  <->  RB0 7セグD 
//              RA7    <->  ┫9       20┣  <-   VDD
//              RA6    <->  ┫10      19┣  <-   Vss
// 水晶発振     RC0    <->  ┫11      18┣  <->  RC7 (USART1 RX)入力
// 水晶発振     RC1    <->  ┫12      17┣  <->  RC6 (USART1 TX)出力
// 　　　　　　 RC2    <->  ┫13      16┣  <->  RC5 SW2入力
// WROOM-02のEN RC3    <->  ┫14      15┣  <->  RC4 SW1入力
//                          ┗━━━━━┛

#include <xc.h>
#include <string.h>
#include <stdio.h>

//設定項目ここから************************************************************
#define WIFISSID1 ""    //WIFIのSSIDを指定 (SW1用)
#define WIFIPSWD1 ""    //WIFIのパスワードを指定

#define WIFISSID2 ""    //WIFIのSSIDを指定 (SW2用)
#define WIFIPSWD2 ""    //WIFIのパスワードを指定

#define NTPSRVADD ""    //NTPサーバのアドレス  例："ntp.nict.jp"

#define NTP_HOUR 5      //時刻合わせする時間 0～23 時
#define NTP_MIN  30     //時刻合わせする時間 0～59 分
#define NTP_SEC  5      //時刻合わせする時間 0～59 秒

#define TIME_ZONE 9     //タイムゾーン(日本は9)
//設定項目ここまで************************************************************


//ピンアサイン
#define LED1 LATA0  //7セグ左から1桁目のカソード側端子につなげるピン
#define LED2 LATA1  //7セグ左から2桁目のカソード側端子につなげるピン
#define LED3 LATA2  //7セグ左から3桁目のカソード側端子につなげるピン
#define LED4 LATA3  //7セグ左から4桁目のカソード側端子につなげるピン
#define LED5 LATA4  //7セグ左から5桁目(コロン表示)のカソード側端子につなげるピン
#define LEDA LATB3  //7セグA
#define LEDB LATB1  //7セグB
#define LEDC LATB4  //7セグC
#define LEDD LATB0  //7セグD
#define LEDE LATB5  //7セグE
#define LEDF LATB6  //7セグF
#define LEDG LATB2  //7セグG
#define LEDP LATB7  //7セグdp
#define WROOM_EN LATC3  //WROOM-02のENピン
#define LED_EX LATA5    //拡張LED お知らせ用のLEDとして使用
#define SW1 PORTCbits.RC4   //スイッチ1入力 (時刻表示と秒表示の切替えSW)
#define SW2 PORTCbits.RC5   //スイッチ2入力 (手動による時刻修正用SW)

//USART通信の実行時間
#define USART1STARTTIME      (NTP_HOUR*60*60+NTP_MIN*60+NTP_SEC)%86400 //WIFIモジュールON時間
#define USART2CONNECTTIME    (USART1STARTTIME+15)%86400 //15秒後 NTPコネクション開始時間
#define USART3SEND1TIME      (USART1STARTTIME+20)%86400 //20秒後 データ送信時間
#define USART4SEND2TIME      (USART1STARTTIME+30)%86400 //20秒後 データ送信時間
#define USART5DISCONNECTTIME (USART1STARTTIME+40)%86400 //40秒後 NTPコネクション終了時間
#define USART6ENDTIME        (USART1STARTTIME+45)%86400 //45秒後 WIFI切断時間
#define USART7OFFTIME        (USART1STARTTIME+50)%86400 //50秒後 WIFIモジュールOFF時間
#define USARTSTATECLEARTIME  (USART1STARTTIME-(10*60))%86400 //測定開始のためのフラグをONさせる時間
//↑開始タイムの10分前とする(開始タイムに近すぎると時刻修正された際にもう一度ONしてしまう可能性あり)
#define USARTBUFSIZE 200//USARTの受信データのバッファサイズ(Byte)


// コンフィギュレーションの設定
// CONFIG1H
#pragma config FOSC = INTIO67   //内部部クロックを使用する(INTIO67)
#pragma config PLLCFG = OFF     //動作クロックを4倍では動作させない(PLLCFG_OFF)
#pragma config PRICLKEN = OFF   //外部クロックは使用しない(PRICLKEN_OFF)
// CONFIG2L
#pragma config PWRTEN = OFF     //電源ONから後65.6msにプログラムを開始する機能をOFF
#pragma config BORV = 285       //監視電圧は(2.85V)に設定
// CONFIG2H
#pragma config WDTEN = OFF      //ウォッチドッグタイマー無し(WDTEN_OFF)
// CONFIG3H
#pragma config MCLRE = EXTMCLR  //リセットスイッチ有効
// CONFIG5L
#pragma config CP0 = OFF         //コードプロテクト Block 0 (Block 0 (000800-001FFFh) code-protected)
#pragma config CP1 = OFF         //コードプロテクト Block 1 (Block 1 (002000-003FFFh) code-protected)
#pragma config CP2 = OFF         //コードプロテクト Block 2 (Block 2 (004000-005FFFh) code-protected)
#pragma config CP3 = OFF         //コードプロテクト Block 3 (Block 3 (006000-007FFFh) code-protected)

//定数定義
#define _XTAL_FREQ 8000000      //delayで使用  OSCCONで設定した周波数と同じにすること
#define ON  1
#define OFF 0

typedef unsigned char uchar;

//関数プロトタイプ
void WifiApSetting();
void dynamicDisp();
void adjClock();
void setTimeMoji();
void set7segbin(uchar n);
void addTime(unsigned short long d);
void cngTime(unsigned short long tim);
void ledDisp();
void usartStrSend(uchar* str,int len); //USART文字列送信
void usartCharSend(uchar c);            //USART1文字送信
void ntpfunc();
uchar setNtpSecond(unsigned long val);  //NTPサーバから受信したデータを現在時刻の秒データに変換

//グローバル変数
//LEDダイナミック点灯関連
const uchar LiDeviceNum = 5;  //ダイナミック点灯するデバイス数を入力
uchar LiDevice = 0;     //点灯中デバイス番号0～
uchar LiState = 0;      //点灯デバイスのステート 0～2
uchar digitA;           //左の桁の7セグ数値
uchar digitB;           //左から2番目の桁の7セグ数値
uchar digitC;           //右から2番目の桁の7セグ数値
uchar digitD;           //右の桁の7セグ数値
uchar digitE;           //コロンの7セグ数値

//タイマー関連
unsigned short long Time = NTP_HOUR*60*60+NTP_MIN*60+NTP_SEC;  //現在の時間（秒）
                                                    //↑初期値はNTP開始時間
unsigned short long oldTime = 0; //現在の時間が更新されたか比較するための変数
unsigned int T1COUNT;       //Timer1のカウント数設定用
uchar TimeIncFlg = OFF;     //秒を＋1秒する割込みが発生したことを示すフラグ

//スイッチ入力
unsigned int sw1count = 0;  //sw1が押されている間カウントアップする変数
unsigned int sw2count = 0;  //sw2が押されている間カウントアップする変数

//おしらせLED
uchar LedBlinkFlg = OFF;    //点滅したいときにONする
uchar LedLightFlg = OFF;    //点灯したいときにONする 消灯したいときにOFFする
uchar BlinkCounter = 0;     //点滅のためのカウンター

//USART通信関連
uchar urcvbuf[USARTBUFSIZE];//USART受信バッファ
int urcvpoint = 0;          //USART受信バッファの書込み位置
uchar UsartState = 0;       //WIFIモジュールとのUSART通信における状態番号

//USART送信データ本体定義
const uchar senddata[] = {  0x0B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

//USART送信文字列定義
const uchar* cstr[] = {
    "AT+CIPSTART=\"UDP\",\"",   //NTPサーバへコネクション開始 前半
    "\",123,,0\r\n",            //NTPサーバへコネクション開始 後半    
    "AT+CIPSEND=48\r\n",        //データ送信開始コマンド
    &senddata,                  //送信データ本体
    "AT+CIPCLOSE\r\n",          //コネクション終了
    "AT+CWQAP\r\n",             //wifi切断
};


//main
void main(void)
{
    digitA = digitB = digitC = digitD = '0';
    digitE = '2';//コロン表示の初期値は2にしておく
    
    //*******************************************
    //PICの設定ここから
    OSCCON = 0b01100010;    //内臓クロック8MHzに設定 (1周期 Tosc = 0.125usec)
    
    //入出力設定
    ANSELA = 0b00000000;    // ANSA  アナログ デジタル設定 0デジタル 1アナログ
    ANSELB = 0b00000000;    // ANSB  アナログ デジタル設定 0デジタル 1アナログ
    ANSELC = 0b00000000;    // ANSC  アナログ デジタル設定 0デジタル 1アナログ
    TRISA  = 0b00000000;    // RA0-RA7入出力設定、0で出力 1で入力
    TRISB  = 0b00000000;    // RB0-RB7入出力設定
    TRISC  = 0b10110000;    // RC0-RC7入出力設定
    LATA   = 0b00000000;    // 出力ピンの初期化(全てLOWにする)
    LATB   = 0b00000000;    // 出力ピンの初期化(全てLOWにする)
    LATC   = 0b00000000;    // 出力ピンの初期化(全てLOWにする)
    //アナログ入力の設定
    ADCON1 = 0b00000000;    // VDDをリファレンスとする
    ADCON2 = 0b10010101;    // 読取値は右寄せ、A/D変換クロックはFOSC/16、4TAD後に変換開始
    
    //タイマ0の設定(可変時間タイマ)LEDのダイナミック点灯で使用
    //  時間 = (256-TMR0)*OPTION_REGの分周比*クロック周期*1命令に必要なクロック
    //  約20msec = (256-100)*256*0.125usec*4
    T0CON = 0b01000111; //bit7 1=Timer0動作 0=Timer0停止 bit6 1=8bit 0=16bit bit5 0=システム周波数でカウント 1=6番PIN入力でカウント bit2-0 000=プリスケール1:2～111=1:256
    TMR0 = 100;     //TMR0カウント値再設定 256-100回(アップカウンタ)
    TMR0IF = 0;     //割込みフラグを初期化
    TMR0IE = 1;     //タイマ0割込み許可
    //TMR0IE = 0;     //タイマ0割込み禁止

    //タイマ1の設定(外部水晶発信子クロック入力)
    T1CON = 0b10011110;
    //    bit7-6でTimer1のクロックソースを選択します
    //      00 = Fosc/4    01 = Fosc    10 = 外部端子(TxCKI又はSOSCI/SOSCO)から入力
    //    bit5-4でプリスケーラの設定を行います
    //      00 = 1：1    01 = 1：2    10 = 1：4    11 = 1：8
    //    bit1  １ = TMR1レジスタを16bitで読書きする  ０ = TMR1レジスタを8bitで読書きする
    //    bit0  １ = TIMER1を動作させる  ０ = TIMER1を動作させない
    T1COUNT = 49151;
    TMR1H = (T1COUNT >> 8);
    TMR1L = (T1COUNT & 0x00ff);
    //49152の計算方法
    //   ((1/水晶周波数)xクロックソース)xプリスケーラ設定値=タイマの1カウントの時間
    //   ((1/32.768kHz)*1)2=0.00006103515625sec   1secのタイマにしたいから 1s/0.00006103515625s=16384カウント
    //   よって65535-16384=49151がタイマでカウントする数値
    TMR1IF = 0; //割込みフラグを初期化
    TMR1IE = 1; //タイマ1割込み有効  1=ON 0=OFF
    
    //USARTの設定  ボーレート115200
    TXSTA1 = 0b00100100;// 送信情報設定：非同期モード 8ビットデータ ノンパリティ //SYNC=0;BRGH=1;
    RCSTA1 = 0b10010000;// 受信情報設定 
    BAUDCON1 = 0b01001000;//BRG16=1;    
    SPBRGH1 = 0;
    SPBRG1 = 16;
    RC1IF = 0;       // USART割込みフラグの初期化
    RC1IE = 1;       // USART割込みを有効にする
    
    PEIE = 1;   //周辺装置割り込みを許可する
    GIE = 1;    //全体割込み許可
    
    //PICの設定ここまで
    //*******************************************

    
    //約1秒間待つ
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    __delay_ms(98);
    
    //起動時にSW1とSW2が押されていたら
    //WIFIアクセスポイント設定を行う
    WifiApSetting();
    
    TMR0ON=1;   //ダイナミック点灯のタイマ開始
    TMR1ON=1;   //リアルタイムクロック開始
        
    //mainループ
    while(1)
    {
        //1秒割込みが発生した？
        if(TimeIncFlg == ON)
        {
            addTime(1); //時刻を1秒すすめる
            TimeIncFlg = OFF;
        }
        
        //7セグのダイナミック点灯
        dynamicDisp();
        
        //LED_EX おしらせLEDの点灯・点滅処理
        ledDisp();
        
        //ボタン操作による時刻修正
        adjClock();
        
        //NTP時刻取得処理
        ntpfunc();
        
    }
}

//WIFIアクセスポイント設定を行う
void WifiApSetting()
{
    //スイッチ状態をローカル変数に取込む
    uchar sw1 = SW1;
    uchar sw2 = SW2;
    
    //どちらかのスイッチが押されているとき SWはOFFで押されている状態
    if(sw1 == OFF || sw2 == OFF)
    {
        //おしらせLED点灯
        LED_EX = ON;
        //WifiモジュールをON
        WROOM_EN = ON;
        //少し待つ
        for(int i=0; i < 50; i++)
        {
            __delay_ms(98);
        }
        //モードをWifi子機モードにする
        usartStrSend("AT+CWMODE=1\r\n",13);
        //少し待つ
        for(int i=0; i < 10; i++)
        {
            __delay_ms(98);
        }
        if(sw1 == OFF)
        {
            //SW1が押されているとき
            usartStrSend("AT+CWJAP=\"",10);
            usartStrSend((uchar*)WIFISSID1,strlen(WIFISSID1));
            usartStrSend("\",\"",3);
            usartStrSend((uchar*)WIFIPSWD1,strlen(WIFIPSWD1));
            usartStrSend("\"\r\n",3);
        }else{
            //SW2が押されているとき
            usartStrSend("AT+CWJAP=\"",10);
            usartStrSend((uchar*)WIFISSID2,strlen(WIFISSID2));
            usartStrSend("\",\"",3);
            usartStrSend((uchar*)WIFIPSWD2,strlen(WIFIPSWD2));
            usartStrSend("\"\r\n",3);    
        }
    }
}

//7セグをダイナミック点灯させる
void dynamicDisp()
{
        
    //時刻を各桁ごとの文字に変換しdigitA～digitEに代入
    setTimeMoji();

    //消灯処理
    if(LiState == 0)
    {
        //全LED OFF状態
        LEDA = LEDB = LEDC = LEDD = LEDE = LEDF = LEDG = LEDP = OFF;
        LED1 = LED2 = LED3 = LED4 = LED5 = OFF;
        return;
    }
    
    //点灯処理
    //  (消灯の割り込み時間が短すぎると一つの前のデバイスがうまく
    //     OFFされないこともあるので前のデバイスのOFFもここに入れておく)
    switch(LiDevice)
    {
        case 0:
            LED5 = OFF;
            set7segbin(digitA);
            LED1 = ON;
            break;
        case 1:
            LED1 = OFF;
            set7segbin(digitB);
            LED2 = ON;
            break;
        case 2:
            LED2 = OFF;
            set7segbin(digitC);
            LED3 = ON;
            break;
        case 3:
            LED3 = OFF;
            set7segbin(digitD);
            LED4 = ON;
            break;
        case 4:
            LED4 = OFF;
            set7segbin(digitE);
            LED5 = ON;
            break;
    }
}


//現在のTimeから各桁の数字文字を計算します
void setTimeMoji()
{
    unsigned short long ti = Time;
    uchar h2,h1,m2,m1,s2,s1;

    if(oldTime == ti)
    {
        return;
    }
    
    oldTime = ti;
    
    //初期化
    h2 = h1 = m2 = m1 = s2 = s1 = 0;

    //XX時の各桁の数値を求める
    while(ti >= 3600){
        ti -= 3600;
        h1++;
        if(h1 == 10)
        {
            h1 = 0;
            h2++;
        }
    }
    //XX分の各桁の数値を求める
    while(ti >= 60){
        ti -= 60;
        m1++;
        if(m1 == 10)
        {
            m1 = 0;
            m2++;
        }
    }
    //XX秒の各桁の数値を求める
    while(ti >= 10)
    {
        ti -= 10;
        s2++;
    }
    s1=ti;

    //各桁の数値が決まったのでグローバル変数へ代入
    if(SW1 == ON)
    {
        if(h2 == 0)
        {
            digitA = '/';//時の頭の0は非表示にする
        }
        else
        {
            digitA = h2 + '0';
        }
        digitB = h1 + '0';
        digitC = m2 + '0';
        digitD = m1 + '0';  
    }
    else
    {    
        digitA = m2 + '0';
        digitB = m1 + '0';
        digitC = s2 + '0';
        digitD = s1 + '0';
    }
}

//数値を渡すと7セグのピンに出力します
void set7segbin(uchar n)
{
    switch(n)
    {
        case '0':
            LEDA = 1;
            LEDB = 1;
            LEDC = 1;
            LEDD = 1;
            LEDE = 1;
            LEDF = 1;
            LEDG = 0;
            break;
        case '1':
            LEDA = 0;
            LEDB = 1;
            LEDC = 1;
            LEDD = 0;
            LEDE = 0;
            LEDF = 0;
            LEDG = 0;
            break;
        case '2':
            LEDA = 1;
            LEDB = 1;
            LEDC = 0;
            LEDD = 1;
            LEDE = 1;
            LEDF = 0;
            LEDG = 1;
            break;
        case '3':
            LEDA = 1;
            LEDB = 1;
            LEDC = 1;
            LEDD = 1;
            LEDE = 0;
            LEDF = 0;
            LEDG = 1;
            break;
        case '4':
            LEDA = 0;
            LEDB = 1;
            LEDC = 1;
            LEDD = 0;
            LEDE = 0;
            LEDF = 1;
            LEDG = 1;
            break;
        case '5':
            LEDA = 1;
            LEDB = 0;
            LEDC = 1;
            LEDD = 1;
            LEDE = 0;
            LEDF = 1;
            LEDG = 1;
            break;
        case '6':
            LEDA = 1;
            LEDB = 0;
            LEDC = 1;
            LEDD = 1;
            LEDE = 1;
            LEDF = 1;
            LEDG = 1;
            break;
        case '7':
            LEDA = 1;
            LEDB = 1;
            LEDC = 1;
            LEDD = 0;
            LEDE = 0;
            LEDF = 0;
            LEDG = 0;
            break;
        case '8':
            LEDA = 1;
            LEDB = 1;
            LEDC = 1;
            LEDD = 1;
            LEDE = 1;
            LEDF = 1;
            LEDG = 1;
            break;
        case '9':
            LEDA = 1;
            LEDB = 1;
            LEDC = 1;
            LEDD = 1;
            LEDE = 0;
            LEDF = 1;
            LEDG = 1;
            break;
        case '/'://空欄 全部消灯
            LEDA = 0;
            LEDB = 0;
            LEDC = 0;
            LEDD = 0;
            LEDE = 0;
            LEDF = 0;
            LEDG = 0;
            break;
        case '-'://マイナス記号
            LEDA = 0;
            LEDB = 0;
            LEDC = 0;
            LEDD = 0;
            LEDE = 0;
            LEDF = 0;
            LEDG = 1;
            break;
        default://不正値なのでEを表示
            LEDA = 1;
            LEDB = 0;
            LEDC = 0;
            LEDD = 1;
            LEDE = 1;
            LEDF = 1;
            LEDG = 1;
            break;
    }

}

//時刻修正 （スイッチ操作で現在時刻を変更する）
//スイッチ1が押されてないときに、スイッチ2を押すと時刻を1分進めます
//スイッチ1を押している間、さらにスイッチ2を押すと秒を0にします
void adjClock()
{
    //NTPで時刻合わせ中は操作不可のため抜ける
    if(WROOM_EN == ON)
        return;
    
    //スイッチ押下状態をチェック
    //スイッチ1が押されてる？（SW押下時OFF）
    if(SW1 == OFF)
    {
        //スイッチ1が押されているとき 秒を0合わせモード
        //スイッチ2が押された？（SW押下時OFF）
        if(SW2 == OFF)
        {
            //00秒にする
            Time = Time - Time%60;
            //割込みの設定をリセット
            TMR1H = (T1COUNT >> 8);
            TMR1L = (T1COUNT & 0x00ff);
        }
    }
    else   
    {
        //スイッチ1が押されていないとき 時刻合わせモード
        //スイッチ2が押された？（SW押下時OFF）
        if(SW2 == OFF)
        {
            //スイッチ2が押されたらカウンタをUP
            //   (ただし、30000になったら29000に戻し、
            //      押し続けると残り1000カウントで押下がリピートされる)
            if(sw2count < 30000){
                sw2count++;
            }
            else
            {
                sw2count = 29000;    
            }
        }
        else
        {
            //スイッチ2が離されたらカウンタを0リセット
            sw2count = 0;
        }

        //スイッチがある程度押された（チャタリング対策＆長押し判定）
        if(sw2count == 2000 || sw2count==30000)
        {
            addTime(60);    //60秒つまり1分進める
        }
    }
}

//引数で指定された秒数時刻を進めます。24時を超えると0時に戻します
void addTime(unsigned short long d)
{
    Time += d;

    //24:00:00は00:00:00に戻す
    if(Time >= 86400)
    {
        Time -= 86400;
    }
}

//お知らせ用LEDの点滅、点灯処理
void ledDisp()
{
    if(LedBlinkFlg)
    {
        //点滅のフラグが立っていたら点滅処理
        if(BlinkCounter > 3)
        {
            BlinkCounter = 0;
            LED_EX = ~LED_EX;
        }
    }
    else
    {
        //点滅のフラグが立っていなければ点灯フラグに従う
        LED_EX = LedLightFlg;
    }

}

//USART 文字列送信
//lenはバイト数。指定文字列以上の値にしないよう注意してください。
void usartStrSend(uchar* str,int len)
{
    for(int i = 0; i < len; i++)
    {
        usartCharSend(str[i]);
    }
}

//USART 1文字送信
void usartCharSend(uchar c)
{
    while(TX1IF == 0) ;//送信可能まで待つ
    TX1REG = c;
}

//NTPサーバと通信し時刻を更新する処理
void ntpfunc()
{
    //UsartStateが0のときに時刻合わせを開始します
    //時刻調整をしたら、また時刻合わせの時間だったなんていう場合、
    //またすぐさままた時刻合わせが実行されないように
    //開始時間の10分前くらいに0クリアします。
    if(Time == USARTSTATECLEARTIME && UsartState != 0)
    {
        //通信ステート(UsartState)をクリア
        UsartState = 0;
    }
    
    //WIFIモジュールをON
    if(Time == USART1STARTTIME && UsartState == 0)
    {        
        UsartState++;
        WROOM_EN = ON;//WROOMのENピンをONします
        LedBlinkFlg = ON;
    }
    
    //NTPコネクション開始
    if(Time == USART2CONNECTTIME && UsartState == 1)
    {
        UsartState++;
        usartStrSend((uchar*)cstr[0],strlen(cstr[0]));
        usartStrSend((uchar*)NTPSRVADD,strlen(NTPSRVADD));
        usartStrSend((uchar*)cstr[1],strlen(cstr[1]));
    }
    
    //NTPへデータ送信前に送信バイト数を指定します
    if(Time == USART3SEND1TIME && UsartState == 2)
    {
        UsartState++;
        usartStrSend((uchar*)cstr[2],strlen(cstr[2]));
    }
    
    //NTPへデータ送信 データ本体を送信します
    if(Time == USART4SEND2TIME && UsartState == 3)
    {
        UsartState++;
        usartStrSend((uchar*)cstr[3],48);
        urcvpoint = 0;
    }
    
    //NTPコネクション終了
    if(Time == USART5DISCONNECTTIME && UsartState == 4)
    {
        UsartState++;
        usartStrSend((uchar*)cstr[4],strlen(cstr[4]));
    }
    
    //WIFI切断
    if(Time == USART6ENDTIME && UsartState == 5)
    {
        UsartState++;
        usartStrSend((uchar*)cstr[5],strlen(cstr[5]));
    }
    
    //WIFIモジュールOFF と 受信した時刻に合わせる
    if(Time == USART7OFFTIME && UsartState == 6)
    {
        //受信した時刻の文字列(バイト列)を数値として扱えるように
        //バイト列を並びかえにunionを利用します
        union NtpTimeData
        {
            uchar uchardata[4];
            unsigned long ulongdata;
        }ntptimedata;
        
        UsartState++;
        WROOM_EN=OFF; //WROOMのENピンをOFFします
        LedBlinkFlg = OFF;
        
        //受信文字列の決められた位置に時刻データがはいっている（はず）
        ntptimedata.uchardata[3]=urcvbuf[78];
        ntptimedata.uchardata[2]=urcvbuf[79];
        ntptimedata.uchardata[1]=urcvbuf[80];
        ntptimedata.uchardata[0]=urcvbuf[81];
        //受信バッファの時刻を部分を0クリア
        urcvbuf[78] = urcvbuf[79] = urcvbuf[80] = urcvbuf[81] = 0;
    
        LedLightFlg = setNtpSecond(ntptimedata.ulongdata);
    }
}

//NTPサーバから受信したデータを現在時刻の秒データに変換
//処理を簡略化するために受信データのエラーチェックは簡単に済ませています
//2036年問題により2036年以降はこの処理では時刻合わせができませんので
//処理を書き換える必要があります。
uchar setNtpSecond(unsigned long val)
{
    //valが2015年以前の値だったら不正とみなし抜ける
    if(val<3626640000) //(2015-1900)*365*24*60*60
        return ON;
    
    while(val>=31536000) //365*24*60*60
        val-=31536000;
    while(val>=2592000) //30*24*60*60
        val-=2592000;

    //タイムゾーンの加算と測定時刻と計算時刻の差異の減算
    val = val + TIME_ZONE*3600 + USART7OFFTIME - USART4SEND2TIME;

    while(val>=86400) //24*60*60
        val=val-86400;

    Time = (unsigned short long)val;
    return OFF;
}

//割込み処理
void interrupt ISR()
{

    //タイマ0のタイムアップ ダイナミック点灯用（割込み間隔は可変してます）
    if (TMR0IF) {

        //ステート番号を進める処理
        LiState++;
        if (LiState > 1)
        {
            //最後のステートまで行ったので次のデバイスの最初のステートにする
            LiState = 0;
            LiDevice++;
            if(LiDevice >= LiDeviceNum)
            {
                LiDevice = 0;
                BlinkCounter++;
            }
        }

        //次の割込みをステートに合わせた割込み時間にする
        switch(LiState)
        {
            case 0:
                TMR0=253;    //255-253= 2カウントだけ非表示
                break;
            case 1:
                TMR0=230;    //255-230= 15カウント表示
                break;
        }

        TMR0IF=0;//タイムアップフラグをクリア
    }

    //タイマ1のタイムアップ リアルタイムクロック用
    if (TMR1IF) {
        
        TimeIncFlg = ON;
          
        //次のタイマのカウント        
        TMR1H = (T1COUNT >> 8);
        TMR1L = (T1COUNT & 0x00ff);
        TMR1IF = 0; //タイムアップフラグをクリア
    }

    //USART 受信
    if(RC1IF)
    {
        uchar dammy;
        if(UsartState == 4) //受信データ待ちのときだけバッファに格納
        {
            //バッファオーバーで他エリアを壊さないようにしておく
            if(urcvpoint >= USARTBUFSIZE - 1)
            {
                urcvpoint = 0;
            }
            urcvbuf[urcvpoint++] = RC1REG;//受信バッファに格納
        }
        else
        {
            dammy = RC1REG; //受信データの読捨て
        }
        RC1IF = 0; //タイムアップフラグをクリア
    }

}



